%{
/**********************************************************************
 errLog.lex: error log parser. Reads an ascii error log and extracts
             error class or category that defines the type of device
             that generated the error and the time when the error occurred.
	     This information is written on a file which is going to be used
             by the DFT program.
 Author: Carol Hernandez
 Followed by Ying : Dec. 18, 1997
 Comments:
	     The error classes identified are architecture independent and 
             are defined as hardware related and software related errors in:
		 /usr/sys/include/dec/binlog/binlog.h
	     If more details about the device that caused the error are needed,
	     the subid_type, subid_ctldevtype, subid_num, and/or subid_unitnum
	     can be also obtained from the error log. I am not sure which 
	     fields in the error log correspond to those features. Maybe 
	     SYSTYPE, CLASS, SUBSYSTEM. All these fields are optional, may
	     or may not apply to the device causing the error, and are 
	     architecture dependent. The definitions for alpha and other
	     digital machines are given in /usr/sys/include/dec/binlog/errlog.h
*****************************************************************************/
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>

int       code;
char      *date;
struct    tm t;
time_t    errTime;
FILE      *file, *out_fd;

extern void sendErrorInfo(int code, time_t errTime);

%}

%s ENTRY
%s CLASS
%s TIMESTAMP

%%

^\n        { ; }
^[ \t]*\n    ; /* ignore blank lines */
[ \t]+     ; /* ignore white spaces, tabs */
^\**\n |
^\*.*\n    { BEGIN ENTRY; }

<ENTRY>"OS EVENT TYPE"	{ BEGIN CLASS; }
<ENTRY>"OCCURRED/LOGGED ON"	{  BEGIN TIMESTAMP;  }
<CLASS>[1-2][0-9][0-9]\.  { 
	   yytext[yyleng-1]='\0';
	   code = atoi(yytext);
	   switch (code) {
	   case 100: 
		code = 0;  /* CPU */
		break;
	   case 101: 
		code = 1;  /* Memory */
		break;
	   case 102: 
		code = 2;  /* Disk */
		break;
	   case 103: 
		code = 3;  /* Tapes */
		break;
	   case 104: 
		code = 4;  /* Device Controllers */
		break;
	   case 105: 
		code = 5;  /* Adapters */
		break;
	   case 106: 
		code = 6;  /* Buses */
		break;
	   case 107: 
		code = 7;  /* Stray Interrupts */
		break;
	   case 108: 
		code = 8;  /* Console Events */
		break;
	   case 109: 
		code = 9;  /* Stack Dumps */
		break;
	   case 198: 
		code = 10;  /* SWXCR RAID Controller Event */
		break;
	   case 199: 
		code = 11;  /* SCSI CAM Events */
		break;
	   case 201: 
		code = 12;  /* CI port-to-port driver event */
		break;
	   case 202: 
		code = 13;  /* System Communications Services */
		break;
           }
	   BEGIN ENTRY;
       }

<TIMESTAMP>.+\n		{
	   date = strdup(yytext);
	   if (strptime(date, "%a %b %d %T %Y", &t) == NULL) {
		printf("strptime: conversion of %s failed\n", date);
		exit(1);
	   }
	   errTime = mktime(&t);
	   if (errTime==-1) {
		printf("mktime:conversion from struct tm to time_t failed\n");
		exit(1);
	   }
	   sendErrorInfo(code, errTime);
	   BEGIN 0;
	  
     }	   

.   { ; }

%%






