/*************************************************************
  errorMonitor.c
  Author: Carol Hernandez
  Followed by Ying : December 20, 1997

  This program reads the binary error log using the shell command
  "/usr/sbin/uerf -fbinary_file -c err > errlog.txt"
  Then it parses the ascii version of the binary error log (errlog.txt)
  to get the error category or device that caused the error and
  the time when the error occurred. This information is communicated
  through a socket to the trendAnalysis module. 
  (errorMonitor is the server for trendAnalysis)

  The binary error log is checked ###every five## minutes for ##new## entries.
  The path for the binary error log is: /usr/adm/binary.errlog. The 
  location and name of the file is specified in the binlog.conf file
  found in /etc/binlog.conf

  The command uerf reads the binary file specified in binlog.conf
  by default. To read another file the option -ffile_name is used. 
  See man uerf for more deatils about the command uerf.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <stdio.h>

#define ADDRESS    "mysocket"    /* addr to connect */
#define INTERVAL  300 

FILE *errLog;
char errLogFile[] = "errlog.txt";

extern FILE *yyin;
extern int yylex();

FILE *fp;
int fromlen;
int i, s, ns, len;
struct sockaddr_un saun, fsaun;

time_t prev_time;

#include "OS_API_Translater.c"

void socketInit()
{
/* Socket initialization - connection establishment */

    /* Get a socket to work with. This socket will be in the Unix domain,
       and will be a stream socket
    */
    if ( (s=socket(AF_UNIX, SOCK_STREAM, 0)) < 0 ) {
       perror("client: socket");
       exit(1);
    }

    /* Create the address we will be connecting to */
    saun.sun_family = AF_UNIX;
    strcpy(saun.sun_path, ADDRESS);

    /* Try to bind the address to the socket. We unlink the name first
       so that the bind won't fail.
       The third argument indicates the "length" of the structure,
       not just the length of the socket name.
    */
    unlink(ADDRESS);
    len = sizeof(saun.sun_family) + strlen(saun.sun_path);
    if ( bind(s, (struct sockaddr *)&saun, len) < 0 ) {
       perror("server:bind");
       exit(1);
    }
    
    /* Listen on the socket. */
    if ( listen(s, 5) < 0 ) {
       perror("server:listen");
       exit(1);
    }

    /* Accept connections. When we accept one, ns will be connected
       to the client. fsaun will contain the address of the client.
    */
    if ( (ns=accept(s, (struct sockaddr *)&fsaun, &fromlen)) < 0 ) {
       perror("server:accept");
       exit(1);
    }

    /* We'll use stdio for reading the socket */
    fp = fdopen(ns, "r");
}

void sendErrorInfo(int code, time_t errTime)
{
    char  strs[30];
#if 0
    printf("got: %d %d from parser, sending to trend analysis module\n",
	    code, errTime);
#endif
    sprintf(strs, "%d %d\n", code, errTime);
    send(ns, strs, strlen(strs), 0);
}

main()
{
    struct tm *t;

    prev_time = time(NULL);
/* For debug purposes set the prev_time to one month before current month */
#ifdef DEBUG
    t = localtime(&prev_time);
    t->tm_sec=t->tm_min=t->tm_hour=0;
    t->tm_mday=1;
    t->tm_mon-=1;
    prev_time = mktime(t);
#endif
    socketInit();
    while (1) {

      periodic_task();   
      errLog = fopen(errLogFile, "r");
      if (!errLog) {
	 fprintf(stderr, "Could not open error log: %s\n", errLogFile);
	 exit(1);
      }/* end of if(errlog)  */
      yyin = errLog;
#ifdef DEBUG
      printf("about to start parsing\n");
#endif
      yylex();
#ifdef DEBUG
      printf("end of parsing\n");
#endif
      fclose(yyin);
      sleep(INTERVAL);
    } /* end of while loop  */

    close(s);
}



