/**********************************************************
  Reliability API
  Author: Carol Hernandez
  Followed by Ying: Dec. 21, 1997
  
  float getAvailability(): 
     Computes the system's availability at time of request
  int   deviceStatus(int deviceId): 
     Returns the status of the device by applying the DFT rules
     to the error log for that device. Returns 1 if a warning 
     has been issued and 0 if no warning has been issued.
     A warning means that the spatial and temporal distribution
     of errors predict a possible failure of the device.
  int   lastWarning(int deviceId):
     Returns the time of the last warning or -1 if no warning has 
     issued for that device.
 ***********************************************************/

#include "reliabilityAPI.h"

float getAvailability(time_t current_time) 
{
   time_t  r_time, d_time, totalTime;
   float   availability;
  
   /* Get Time Reference */
   timeRef = fopen(timeRefFile, "r");
   if (!timeRef) {
       fprintf(stderr, "Could not open timeRef file: %s\n", timeRefFile);
       exit(1);
   }   
   fscanf(timeRef, "%d", &r_time);
   fclose(timeRef);

   /* Get Down Time */
   downTime = fopen(downTimeFile, "r");
   if (!downTime) {
       fprintf(stderr, "Could not open down time file: %s\n", downTimeFile);
       exit(1);
   }
   fscanf(downTime, "%d", &d_time);  
   fclose(downTime);

   totalTime = current_time - r_time;  

   /* availability = upTime (total-down) / total */
   availability = (float) (totalTime - d_time)/(float)(totalTime);
   return(availability);
}

int   deviceStatus(int deviceId) 
{
   FILE *fd;
   char str[20];
   int warning;

   sprintf(str, "%s.dft", fruName[deviceId]);
   fd = fopen(str, "r");
   if (!fd) {
       printf("Error opening warning file %s\n", str);
       exit(1);
   }
   fscanf(fd, "%d", &warning);
   return(warning);
}

int   lastWarning(int deviceId)
{
   FILE *fd;
   char str[20];
   int t;

   sprintf(str, "%s.out", fruName[deviceId]);
   fd = fopen(str, "r");
   if (!fd) {
       printf("Error opening warning file %s\n", str);
       exit(1);
   }
   fscanf(fd, "%d", &t);
   return(t);
}


