/*************************************************************
  trendAnalysis.c
  Author: Carol Hernandez
  Followed by Ying : December 20, 1997

  This program reads the error info from a socket, filters the
  error entries, records the error events and tries to predict if 
  the device that caused the error will fail in the near future.
  ( trendAnalysis is the client of errorMonior )

  Files created:
  devicename.out : history of warnings issued for that device
                   format: rule fired - time of warning
  devicename.dft : time of last warning issued for that device
                   format: time of last warning

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <stdio.h>

#define ADDRESS    "mysocket"    /* addr to connect */

#include "dft.h"

struct FRU fru_devices[NDEVICES];


FILE *fp;
int s, len;
struct sockaddr_un saun;

void socketInit()
{
/* Socket initialization - connection establishment */

    /* Get a socket to work with. This socket will be in the Unix domain,
       and will be a stream socket
    */
    if ( (s=socket(AF_UNIX, SOCK_STREAM, 0)) < 0 ) {
       perror("client: socket");
       exit(1);
    }

    /* Create the address we will be connecting to */
    saun.sun_family = AF_UNIX;
    strcpy(saun.sun_path, ADDRESS);
    FILE *fd;    
    /* Try to connect to the address. For this to succeed, the server 
       must already have bound this address, and must have issued a
       listen() request.
       The third argument indicates the "length" of the structure,
       not just the length of the socket name.
    */
    len = sizeof(saun.sun_family) + strlen(saun.sun_path);
    if ( connect(s, (struct sockaddr *)&saun, len) < 0 ) {
       perror("client:connect");
       exit(1);
    }

    /* We'll use stdio for reading the socket */
    fp = fdopen(s, "r");
}

void deviceRecordInit()
{
   FILE *fd;
   char str[20];
   int i;

   for (i=0; i < NDEVICES; i++) {
        initFRU(fru_devices[i], fruType[i], fruName[i]);

    /* Initialize warning File to 0 (no warning has been issued) */
	sprintf(str, "%s.dft", fruName[i]);
	fd = fopen(str, "w");
	if (!fd) {
	   printf("Error opening warning file %s\n", str);
	   exit(1);
	}
	fprintf(fd, "0");
	fclose(fd);

	/* Create files that keep warning history */
	sprintf(str, "%s.out", fruName[i]);
	fd = fopen(str, "w");
	if (!fd) {
	   printf("Error creating warning file %s\n", str);
	   exit(1);
	}
	fclose(fd);
   }
}

FILE * openFile(char *fileName)
{
   FILE *fd;

   fd = fopen(fileName, "a");
   if (!fd) {
      fprintf(stderr, "Could not open file for append: %s\n", fileName);
      exit(1);
   }
   return fd;
}

void sendMail(char *stime, char *device)
{

}

void processErrorInfo(FRU_Type type, time_t timeStamp)
{
    int i, warning;
    char str[20];
    time_t tt;
    FILE *fd;    

    /****************
        for dimo purpose chunck below
                      ******************/
    int err;
    char buff[60];
    char subject[]="warning";
    char dest[]="yshi";

    /***************
        for dimo purpose chunck ends
                      *******************/



    if (type >= NDEVICES) {
	printf("Unrecognized FRU: %d\n", type);
	exit(1);
    }
    tt = time(NULL);
    warning=recordError(timeStamp, fru_devices[type]);
    if (warning) {
       /* report rule fired and time of warning on device files */
       for (i=0; i<NRULES; i++) {
	   if (fru_devices[type].rulesFired[i]) {
	      sprintf(str, "%s.out", fru_devices[type].name);
	      fd = openFile(str);
	      fprintf(fd, "i=%d\t%s\n",i, ctime(&tt));

              if (i==(NRULES-1)) fprintf(fd,"\n\n\n");

	      fclose(fd);
                
                 /****************
                        for dimo purpose chunck below
                                        *******************/

              sprintf(buff, "mail %s < /home/yshi/amar/src/%s ", dest, str);
              err= system(buff);
              if (err==-1) {
                 printf("Couldn't execute shell command: %s\n", buff);
                 exit(1);
              }


                 /****************
                        for dimo purpose chunck ends
                                        *******************/

	      fru_devices[type].rulesFired[i]=0;  /* Reset rule */
	   }
           else {
              sprintf(str, "%s.out", fru_devices[type].name);
              fd = openFile(str);

              if (i==(NRULES-1)) fprintf(fd,"\n\n\n");

              fclose(fd);
           }
       }

       /* record time of warning */
       sprintf(str, "%s.dft", fru_devices[type].name);
       fd = fopen(str, "w");
       if (!fd) {
	  printf("Error opening %s\n", str);
	  exit(1);
       }
       fprintf(fd, "%d", tt);
       fclose(fd);

       /* send mail to administrator */
       sendMail(ctime(&tt), fru_devices[type].name);
     }
}

main()
{
    int n;
    int type, t;

    socketInit();
    deviceRecordInit();
    while (1) {
      n = fscanf(fp, "%d %d\n", &type, &t);
      if (n!=EOF) {
#ifdef DEBUG
         printf("Client received: %d %d\n", type, t);
#endif
         processErrorInfo((enum FRU_Type)type, t);
      }
    }
    close(s);
}


